from typing import Final


# ====== CC: Common Table Column ======
class CC:
    built_time: Final[str] = 'built_time'

    description_id: Final[str] = 'description_id'

    formatted_description: Final[str] = 'formatted_description'

    info: Final = 'info'

    last_used_time: Final[str] = 'last_used_time'
    length: Final[str] = 'length'

    plain_description: Final[str] = 'plain_description'

    text: Final[str] = 'text'

    used_count: Final[str] = 'used_count'
