from typing import Final


# ====== CFn: Common Custom Function ======
class CFn:
    array_sort: Final[str] = 'array_sort'

    bigint_2_id: Final[str] = 'bigint_2_id'

    count_jsonb_keys: Final[str] = 'count_jsonb_keys'

    id_2_bigint: Final[str] = 'id_2_bigint'
    id_2_int: Final[str] = 'id_2_int'
    if_null_2_empty_bigint_array: Final[str] = 'if_null_2_empty_bigint_array'
    int_2_id: Final[str] = 'int_2_id'
    is_null: Final[str] = 'is_null'

    jsonb_add_key_value_elements: Final[str] = 'jsonb_add_key_value_elements'
    jsonb_array_2_bigint_array: Final[str] = 'jsonb_array_2_bigint_array'
    jsonb_array_2_int_array: Final[str] = 'jsonb_array_2_int_array'
    jsonb_array_2_smallint_array: Final[str] = 'jsonb_array_2_smallint_array'
    jsonb_array_2_text_array: Final[str] = 'jsonb_array_2_text_array'
    jsonb_remove_key: Final[str] = 'jsonb_remove_key'

    jsonb_id_array_2_bigint_array: Final[str] = 'jsonb_id_array_2_bigint_array'
