from typing import Final


# ====== CA: Common Attribute ======

class CA:
    Api: Final[str] = "api"
    Auth: Final[str] = "aut"
    Ascending: Final[str] = "asc"  # in SQL

    Check: Final[str] = "chk"
    Click: Final[str] = "clk"
    Code: Final[str] = "cd"
    Collect: Final[str] = "clt"
    Collected: Final[str] = "cltd"
    Color: Final[str] = "col"
    Count: Final[str] = "cnt"
    Create: Final[str] = "ct"
    CreateTime: Final[str] = "crt"

    Data: Final[str] = "dt"
    Descending: Final[str] = "desc"  # in SQL
    Description: Final[str] = "des"
    DescriptionFormatted: Final[str] = "desf"
    DescriptionId: Final[str] = "desid"
    DescriptionPlain: Final[str] = "desp"

    Finished: Final[str] = "fied"

    Head: Final[str] = "hd"
    HeadVersion: Final[str] = "hv"

    Id: Final[str] = "id"
    Info: Final[str] = "inf"

    List: Final[str] = "lt"
    Login: Final[str] = "lgn"

    Name: Final[str] = "na"
    Note: Final[str] = "nt"

    Password: Final[str] = "pw"
    Public: Final[str] = "pb"

    Request: Final[str] = "rqt"
    Result: Final[str] = "ret"

    Signup: Final[str] = "sgu"
    SortBy: Final[str] = "stby"
    SortDirection: Final[str] = "stdr"
    Stars: Final[str] = "strs"
    Start: Final[str] = "str"
    State: Final[str] = "ste"

    Text: Final[str] = "txt"
    Time: Final[str] = "tm"
    TimeZone: Final[str] = "tmz"
    Title: Final[str] = "tt"
    TitleId: Final[str] = "ttid"
    Type: Final[str] = "tp"

    Update: Final[str] = "ud"
    UpdateTime: Final[str] = "udt"
    Used: Final[str] = "used"
    User: Final[str] = "usr"
    UserId: Final[str] = "uid"

    Version: Final[str] = "vn"
    VersionCreate: Final[str] = "vct"
    VersionId: Final[str] = "vid"
    Visited: Final[str] = "vst"
    Visiting: Final[str] = "vstng"

    Word: Final[str] = "w"

    ___Error___: Final[str] = "err"
